@@ -2,4 +2,4 @@
 from .landslide_probability import LandslideProbability
 
 
-__all__ = ['LandslideProbability', ]
+__all__ = ["LandslideProbability"]
