@@ -6,17 +6,17 @@ Created on Mon Oct 19.
 """
 from __future__ import print_function
 
-import landlab
-from landlab import (ModelParameterDictionary, Component, FieldError,
-                     FIXED_VALUE_BOUNDARY)
-
-from landlab.utils.decorators import use_file_name_or_kwds, deprecated
-from landlab.core.model_parameter_dictionary import MissingKeyError
-from landlab.components.flow_routing import (DepressionFinderAndRouter,
-                                             FlowRouter)
-from landlab.grid.base import BAD_INDEX_VALUE
 import numpy as np
 
+import landlab
+from landlab import Component, FieldError, ModelParameterDictionary
+from landlab.components import DepressionFinderAndRouter, FlowAccumulator
+from landlab.core.model_parameter_dictionary import MissingKeyError
+from landlab.grid.base import BAD_INDEX_VALUE
+from landlab.utils.decorators import deprecated, use_file_name_or_kwds
+
+# TODO: this should probably follow Barnes et al., 2014 for max efficiency
+
 
 class SinkFiller(Component):
     """
@@ -37,20 +37,20 @@ class SinkFiller(Component):
     --------
     >>> from landlab import RasterModelGrid
     >>> from landlab import BAD_INDEX_VALUE as XX
-    >>> from landlab.components import FlowRouter, SinkFiller
+    >>> from landlab.components import FlowAccumulator, SinkFiller
     >>> import numpy as np
     >>> lake1 = np.array([34, 35, 36, 44, 45, 46, 54, 55, 56, 65, 74])
     >>> lake2 = np.array([78, 87, 88])
     >>> guard_nodes = np.array([23, 33, 53, 63, 73, 83])
     >>> lake = np.concatenate((lake1, lake2))
-    >>> mg = RasterModelGrid((10, 10), 1.)
+    >>> mg = RasterModelGrid((10, 10))
     >>> z = np.ones(100, dtype=float)
     >>> z += mg.node_x  # add a slope
     >>> z[guard_nodes] += 0.001  # forces the flow out of a particular node
     >>> z[lake] = 0.
     >>> field = mg.add_field('node', 'topographic__elevation', z,
     ...                      units='-', copy=True)
-    >>> fr = FlowRouter(mg)
+    >>> fr = FlowAccumulator(mg, flow_director='D8')
     >>> fr.run_one_step()
     >>> mg.at_node['flow__sink_flag'][mg.core_nodes].sum()
     14
@@ -78,31 +78,24 @@ class SinkFiller(Component):
     >>> mg.at_node['flow__sink_flag'][mg.core_nodes].sum()
     0
     """
-    _name = 'SinkFiller'
 
-    _input_var_names = ('topographic__elevation',
-                        )
+    _name = "SinkFiller"
 
-    _output_var_names = ('topographic__elevation',
-                         'sediment_fill__depth',
-                         )
+    _input_var_names = ("topographic__elevation",)
 
-    _var_units = {'topographic__elevation': 'm',
-                  'sediment_fill__depth': 'm',
-                  }
+    _output_var_names = ("topographic__elevation", "sediment_fill__depth")
 
-    _var_mapping = {'topographic__elevation': 'node',
-                    'sediment_fill__depth': 'node',
-                    }
+    _var_units = {"topographic__elevation": "m", "sediment_fill__depth": "m"}
 
-    _var_doc = {'topographic__elevation': 'Surface topographic elevation',
-                'sediment_fill__depth': 'Depth of sediment added at each' +
-                                        'node',
-                }
+    _var_mapping = {"topographic__elevation": "node", "sediment_fill__depth": "node"}
+
+    _var_doc = {
+        "topographic__elevation": "Surface topographic elevation",
+        "sediment_fill__depth": "Depth of sediment added at each" + "node",
+    }
 
     @use_file_name_or_kwds
-    def __init__(self, grid, routing='D8', apply_slope=False,
-                 fill_slope=1.e-5, **kwds):
+    def __init__(self, grid, routing="D8", apply_slope=False, fill_slope=1.e-5, **kwds):
         """
         Parameters
         ----------
@@ -123,21 +116,24 @@ class SinkFiller(Component):
             The slope added to the top surface of filled pits to allow flow
             routing across them, if apply_slope.
         """
-        if 'flow__receiver_node' in grid.at_node:
-            if (grid.at_node['flow__receiver_node'].size != grid.size('node')):
-                msg = ('A route-to-multiple flow director has been '
-                       'run on this grid. The landlab development team has not '
-                       'verified that SinkFiller is compatible with '
-                       'route-to-multiple methods. Please open a GitHub Issue '
-                       'to start this process.')
+        if "flow__receiver_node" in grid.at_node:
+            if grid.at_node["flow__receiver_node"].size != grid.size("node"):
+                msg = (
+                    "A route-to-multiple flow director has been "
+                    "run on this grid. The landlab development team has not "
+                    "verified that SinkFiller is compatible with "
+                    "route-to-multiple methods. Please open a GitHub Issue "
+                    "to start this process."
+                )
                 raise NotImplementedError(msg)
 
         self._grid = grid
-        if routing is not 'D8':
-            assert routing is 'D4'
+        if routing is not "D8":
+            assert routing is "D4"
         self._routing = routing
-        if ((type(self._grid) is landlab.grid.raster.RasterModelGrid) and
-                (routing is 'D8')):
+        if (type(self._grid) is landlab.grid.raster.RasterModelGrid) and (
+            routing is "D8"
+        ):
             self._D8 = True
             self.num_nbrs = 8
         else:
@@ -167,41 +163,42 @@ class SinkFiller(Component):
         #  2. The input file has an item called 'ELEVATION_FIELD_NAME' *and*
         #     a field by this name exists in the grid.
         try:
-            self._elev = self._grid.at_node['topographic__elevation']
+            self._elev = self._grid.at_node["topographic__elevation"]
         except FieldError:
             try:
-                self.topo_field_name = inputs.read_string('ELEVATION_' +
-                                                          'FIELD_NAME')
+                self.topo_field_name = inputs.read_string("ELEVATION_" + "FIELD_NAME")
             except AttributeError:
-                print('Error: Because your grid does not have a node field')
+                print("Error: Because your grid does not have a node field")
                 print('called "topographic__elevation", you need to pass the')
-                print('name of a text input file or ModelParameterDictionary,')
-                print('and this file or dictionary needs to include the name')
-                print('of another field in your grid that contains your')
-                print('elevation data.')
+                print("name of a text input file or ModelParameterDictionary,")
+                print("and this file or dictionary needs to include the name")
+                print("of another field in your grid that contains your")
+                print("elevation data.")
                 raise AttributeError
             except MissingKeyError:
-                print('Error: Because your grid does not have a node field')
+                print("Error: Because your grid does not have a node field")
                 print('called "topographic__elevation", your input file (or')
-                print('ModelParameterDictionary) must include an entry with')
+                print("ModelParameterDictionary) must include an entry with")
                 print('the key "ELEVATION_FIELD_NAME", which gives the name')
-                print('of a field in your grid that contains your elevation')
-                print('data.')
-                raise MissingKeyError('ELEVATION_FIELD_NAME')
+                print("of a field in your grid that contains your elevation")
+                print("data.")
+                raise MissingKeyError("ELEVATION_FIELD_NAME")
             try:
                 self._elev = self._grid.at_node[self.topo_field_name]
             except AttributeError:
-                print('Your grid does not seem to have a node field called',
-                      self.topo_field_name)
+                print(
+                    "Your grid does not seem to have a node field called",
+                    self.topo_field_name,
+                )
         else:
-            self.topo_field_name = 'topographic__elevation'
+            self.topo_field_name = "topographic__elevation"
         # create the only new output field:
-        self.sed_fill_depth = self._grid.add_zeros('node',
-                                                   'sediment_fill__depth',
-                                                   noclobber=False)
+        self.sed_fill_depth = self._grid.add_zeros(
+            "node", "sediment_fill__depth", noclobber=False
+        )
 
         self._lf = DepressionFinderAndRouter(self._grid, routing=self._routing)
-        self._fr = FlowRouter(self._grid, method=self._routing)
+        self._fr = FlowAccumulator(self._grid, flow_director=self._routing)
 
     def fill_pits(self, **kwds):
         """
@@ -216,7 +213,7 @@ class SinkFiller(Component):
         """
         # added for back-compatibility with old formats
         try:
-            self._apply_slope = kwds['apply_slope']
+            self._apply_slope = kwds["apply_slope"]
         except KeyError:
             pass
         self.original_elev = self._elev.copy()
@@ -228,8 +225,7 @@ class SinkFiller(Component):
         # delete them!
         existing_fields = {}
         spurious_fields = set()
-        set_of_outputs = (set(self._lf.output_var_names) |
-                          set(self._fr.output_var_names))
+        set_of_outputs = set(self._lf.output_var_names) | set(self._fr.output_var_names)
         try:
             set_of_outputs.remove(self.topo_field_name)
         except KeyError:
@@ -240,45 +236,51 @@ class SinkFiller(Component):
             except FieldError:  # not there; good!
                 spurious_fields.add(field)
 
-        self._fr.route_flow()
-        self._lf.map_depressions(pits=self._grid.at_node['flow__sink_flag'],
-                                 reroute_flow=True)
+        self._fr.run_one_step()
+        self._lf.map_depressions(
+            pits=self._grid.at_node["flow__sink_flag"], reroute_flow=True
+        )
         # add the depression depths to get up to flat:
-        self._elev += self._grid.at_node['depression__depth']
+        self._elev += self._grid.at_node["depression__depth"]
         # if apply_slope is none, we're now done! But if not...
         if self._apply_slope:
             # new way of doing this - use the upstream structure! Should be
             # both more general and more efficient
-            for (outlet_node, lake_code) in zip(self._lf.lake_outlets,
-                                                self._lf.lake_codes):
+            for (outlet_node, lake_code) in zip(
+                self._lf.lake_outlets, self._lf.lake_codes
+            ):
                 lake_nodes = np.where(self._lf.lake_map == lake_code)[0]
                 lake_perim = self._get_lake_ext_margin(lake_nodes)
                 perim_elevs = self._elev[lake_perim]
                 out_elev = self._elev[outlet_node]
                 lowest_elev_perim = perim_elevs[perim_elevs != out_elev].min()
                 # note we exclude the outlet node
-                elev_increment = ((lowest_elev_perim-self._elev[outlet_node]) /
-                                  (lake_nodes.size + 2.))
+                elev_increment = (lowest_elev_perim - self._elev[outlet_node]) / (
+                    lake_nodes.size + 2.
+                )
                 assert elev_increment > 0.
-                all_ordering = self._grid.at_node['flow__upstream_node_order']
-                upstream_order_bool = np.in1d(all_ordering, lake_nodes,
-                                              assume_unique=True)
+                all_ordering = self._grid.at_node["flow__upstream_node_order"]
+                upstream_order_bool = np.in1d(
+                    all_ordering, lake_nodes, assume_unique=True
+                )
                 lake_upstream_order = all_ordering[upstream_order_bool]
                 argsort_lake = np.argsort(lake_upstream_order)
-                elevs_to_add = (np.arange(lake_nodes.size, dtype=float) +
-                                1.) * elev_increment
+                elevs_to_add = (
+                    np.arange(lake_nodes.size, dtype=float) + 1.
+                ) * elev_increment
                 sorted_elevs_to_add = elevs_to_add[argsort_lake]
                 self._elev[lake_nodes] += sorted_elevs_to_add
         # now put back any fields that were present initially, and wipe the
         # rest:
         for delete_me in spurious_fields:
-            self._grid.delete_field('node', delete_me)
+            if delete_me in self._grid.at_node:
+                self._grid.delete_field("node", delete_me)
         for update_me in existing_fields.keys():
             self.grid.at_node[update_me][:] = existing_fields[update_me]
         # fill the output field
         self.sed_fill_depth[:] = self._elev - self.original_elev
 
-    @deprecated(use='fill_pits', version=1.0)
+    @deprecated(use="fill_pits", version=1.0)
     def _fill_pits_old(self, apply_slope=None):
         """
 
@@ -328,11 +330,12 @@ class SinkFiller(Component):
             except FieldError:  # not there; good!
                 spurious_fields.add(field)
 
-        self._fr.route_flow()
-        self._lf.map_depressions(pits=self._grid.at_node['flow__sink_flag'],
-                                 reroute_flow=False)
+        self._fr.run_one_step()
+        self._lf.map_depressions(
+            pits=self._grid.at_node["flow__sink_flag"], reroute_flow=False
+        )
         # add the depression depths to get up to flat:
-        self._elev += self._grid.at_node['depression__depth']
+        self._elev += self._grid.at_node["depression__depth"]
         # if apply_slope is none, we're now done! But if not...
         if apply_slope is True:
             apply_slope = self._fill_slope
@@ -347,11 +350,12 @@ class SinkFiller(Component):
             self.lake_nodes_treated = np.array([], dtype=int)
             while unstable:
                 while 1:
-                    for (outlet_node, lake_code) in zip(self._lf.lake_outlets,
-                                                        self._lf.lake_codes):
-                        self._apply_slope_current_lake(apply_slope,
-                                                       outlet_node,
-                                                       lake_code, sublake)
+                    for (outlet_node, lake_code) in zip(
+                        self._lf.lake_outlets, self._lf.lake_codes
+                    ):
+                        self._apply_slope_current_lake(
+                            apply_slope, outlet_node, lake_code, sublake
+                        )
                     # Call the mapper again here. Bail out if no core pits are
                     # found.
                     # This is necessary as there are some configs where adding
@@ -359,29 +363,31 @@ class SinkFiller(Component):
                     self._lf.map_depressions(pits=None, reroute_flow=False)
                     if len(self._lf.lake_outlets) == 0.:
                         break
-                    self._elev += self._grid.at_node['depression__depth']
+                    self._elev += self._grid.at_node["depression__depth"]
                     sublake = True
                     self.lake_nodes_treated = np.array([], dtype=int)
                 # final test that all lakes are not reversing flow dirs
-                all_lakes = np.where(self._lf.flood_status <
-                                     BAD_INDEX_VALUE)[0]
-                unstable = self.drainage_directions_change(all_lakes,
-                                                           self.original_elev,
-                                                           self._elev)
+                all_lakes = np.where(self._lf.flood_status < BAD_INDEX_VALUE)[0]
+                unstable = self.drainage_directions_change(
+                    all_lakes, self.original_elev, self._elev
+                )
                 if unstable:
                     apply_slope *= 0.1
                     sublake = False
                     self.lake_nodes_treated = np.array([], dtype=int)
-                    self._elev[:] = original_elev  # put back init conds
+                    self._elev[:] = self.original_elev  # put back init conds
                     stability_increment += 1
                     if stability_increment == 10:
-                        raise OverflowError('Filler could not find a stable ' +
-                                            'condition with a sloping ' +
-                                            'surface!')
+                        raise OverflowError(
+                            "Filler could not find a stable "
+                            + "condition with a sloping "
+                            + "surface!"
+                        )
         # now put back any fields that were present initially, and wipe the
         # rest:
         for delete_me in spurious_fields:
-            self._grid.delete_field('node', delete_me)
+            if delete_me in self.grid.at_node:
+                self._grid.delete_field("node", delete_me)
         for update_me in existing_fields.keys():
             self.grid.at_node[update_me] = existing_fields[update_me]
         # fill the output field
@@ -393,17 +399,16 @@ class SinkFiller(Component):
         the outlet to the nodes in the lake.
         """
         new_elevs = self._elev.copy()
-        outlet_coord = (self._grid.node_x[outlet_node],
-                        self._grid.node_y[outlet_node])
+        outlet_coord = (self._grid.node_x[outlet_node], self._grid.node_y[outlet_node])
         lake_nodes = np.where(self._lf.lake_map == lake_code)[0]
         lake_nodes = np.setdiff1d(lake_nodes, self.lake_nodes_treated)
-        lake_ext_margin = self._get_lake_ext_margin(lake_nodes)
-        d = self._grid.calc_distances_of_nodes_to_point(outlet_coord,
-                                                        node_subset=lake_nodes)
-        add_vals = slope*d
+        # lake_ext_margin = self._get_lake_ext_margin(lake_nodes)
+        d = self._grid.calc_distances_of_nodes_to_point(
+            outlet_coord, node_subset=lake_nodes
+        )
+        add_vals = slope * d
         new_elevs[lake_nodes] += add_vals
-        self.lake_nodes_treated = np.union1d(self.lake_nodes_treated,
-                                             lake_nodes)
+        self.lake_nodes_treated = np.union1d(self.lake_nodes_treated, lake_nodes)
         return new_elevs, lake_nodes
 
     def _get_lake_ext_margin(self, lake_nodes):
@@ -414,10 +419,10 @@ class SinkFiller(Component):
         if self._D8 is True:
             all_poss = np.union1d(
                 self.grid.active_adjacent_nodes_at_node[lake_nodes],
-                self.grid.diagonal_adjacent_nodes_at_node[lake_nodes])
+                self.grid.diagonal_adjacent_nodes_at_node[lake_nodes],
+            )
         else:
-            all_poss = np.unique(self.grid.active_adjacent_nodes_at_node[
-                lake_nodes])
+            all_poss = np.unique(self.grid.active_adjacent_nodes_at_node[lake_nodes])
         lake_ext_edge = np.setdiff1d(all_poss, lake_nodes)
         return lake_ext_edge[lake_ext_edge != BAD_INDEX_VALUE]
 
@@ -430,30 +435,30 @@ class SinkFiller(Component):
         if self._D8 is True:
             all_poss_int = np.union1d(
                 self._grid.active_adjacent_nodes_at_node[lee],
-                self._grid.diagonal_adjacent_nodes_at_node[lee])
+                self._grid.diagonal_adjacent_nodes_at_node[lee],
+            )
         else:
             all_poss_int = np.unique(self._grid.active_adjacent_nodes_at_node[lee])
         lake_int_edge = np.intersect1d(all_poss_int, lake_nodes)
         return lake_int_edge[lake_int_edge != BAD_INDEX_VALUE]
 
-    def _apply_slope_current_lake(self, apply_slope, outlet_node, lake_code,
-                                 sublake):
+    def _apply_slope_current_lake(self, apply_slope, outlet_node, lake_code, sublake):
         """
         Wraps the _add_slopes method to allow handling of conditions where the
         drainage structure would be changed or we're dealing with a sublake.
         """
         while 1:
             starting_elevs = self._elev.copy()
-            self._elev[:], lake_nodes = self._add_slopes(apply_slope,
-                                                         outlet_node,
-                                                         lake_code)
-            ext_edge = self._get_lake_ext_margin(lake_nodes)
+            self._elev[:], lake_nodes = self._add_slopes(
+                apply_slope, outlet_node, lake_code
+            )
+            # ext_edge = self._get_lake_ext_margin(lake_nodes)
             if sublake:
                 break
             else:
-                if not self.drainage_directions_change(lake_nodes,
-                                                       starting_elevs,
-                                                       self._elev):
+                if not self.drainage_directions_change(
+                    lake_nodes, starting_elevs, self._elev
+                ):
                     break
                 else:
                     # put the elevs back...
@@ -469,18 +474,21 @@ class SinkFiller(Component):
         ext_edge = self._get_lake_ext_margin(lake_nodes)
         if self._D8:
             edge_neighbors = np.hstack(
-                (self.grid.active_adjacent_nodes_at_node[ext_edge],
-                 self.grid.diagonal_adjacent_nodes_at_node[ext_edge]))
+                (
+                    self.grid.active_adjacent_nodes_at_node[ext_edge],
+                    self.grid.diagonal_adjacent_nodes_at_node[ext_edge],
+                )
+            )
         else:
-            edge_neighbors = self.grid.active_adjacent_nodes_at_node[
-                ext_edge].copy()
+            edge_neighbors = self.grid.active_adjacent_nodes_at_node[ext_edge].copy()
         edge_neighbors[edge_neighbors == BAD_INDEX_VALUE] = -1
         # ^value irrelevant
         old_neighbor_elevs = old_elevs[edge_neighbors]
         new_neighbor_elevs = new_elevs[edge_neighbors]
         # enforce the "don't change drainage direction" condition:
         edge_elevs = old_elevs[ext_edge].reshape((ext_edge.size, 1))
-        cond = np.allclose((edge_elevs >= old_neighbor_elevs),
-                           (edge_elevs >= new_neighbor_elevs))
+        cond = np.allclose(
+            (edge_elevs >= old_neighbor_elevs), (edge_elevs >= new_neighbor_elevs)
+        )
         # if True, we're good, the tilting didn't mess with the fr
         return not cond
